@@ -41,7 +41,10 @@
 #define SQL_CREATE_TABLE_GAME \
         "CREATE TABLE IF NOT EXISTS game (" \
                 "name PRIMARY KEY, " \
+                "category, " \
                 "sourcefile, " \
+                "isbios DEFAULT 'no' " \
+                "CHECK (isbios in ('yes', 'no')), " \
                 "runnable DEFAULT 'yes' " \
                 "CHECK (runnable in ('yes', 'no')), " \
                 "cloneof, " \
@@ -144,14 +147,21 @@
         "CREATE TABLE IF NOT EXISTS display (" \
                 "game NOT NULL, " \
                 "type NOT NULL " \
-                "CHECK (type in ('raster', 'vector')), " \
+                "CHECK (type in ('raster', 'vector', 'lcd', 'unknown')), " \
                 "rotate NOT NULL " \
                 "CHECK (rotate in ('0', '90', '180', '270')), " \
                 "flipx DEFAULT 'no' " \
                 "CHECK (flipx in ('yes', 'no')), " \
                 "width, " \
                 "height, " \
-                "refresh NOT NULL);"
+                "refresh NOT NULL, " \
+                "pixclock, " \
+                "htotal, " \
+                "hbend, " \
+                "hbstart, " \
+                "vtotal, " \
+                "vbend, " \
+                "vbstart);"
 
 #define SQL_CREATE_TABLE_CONTROL \
         "CREATE TABLE IF NOT EXISTS control (" \
@@ -181,9 +191,12 @@
 
 #define SQL_CREATE_VIEW_AVAILABLE \
         "CREATE VIEW IF NOT EXISTS available AS " \
-                "SELECT *, getcategory(name) AS category, " \
-                "isfavorite(name) AS favorite FROM game " \
-                "WHERE (romset IN ('good', 'best available'))"
+                "SELECT game.*, bios.description AS bios, " \
+                "isfavorite(game.name) AS favorite FROM game " \
+                "LEFT JOIN (SELECT name, description FROM game WHERE " \
+                "isbios = 'yes') AS bios ON game.romof = bios.name " \
+                "WHERE (romset IN ('good', 'best available') " \
+                "AND isbios = 'no');"
 
 #define SQL_DROP_TABLES \
         "DROP TABLE IF EXISTS mame; " \
@@ -201,7 +214,9 @@
 #define SQL_INSERT_GAME \
         "INSERT INTO game VALUES (" \
                 "@name, " \
+                "@category, " \
                 "@sourcefile, " \
+                "@isbios, " \
                 "@runnable, " \
                 "@cloneof, " \
                 "@romof, " \
@@ -280,7 +295,14 @@
                 "@flipx, " \
                 "@width, " \
                 "@height, " \
-                "@refresh);"
+                "@refresh, " \
+                "@pixclock, " \
+                "@htotal, " \
+                "@hbend, " \
+                "@hbstart, " \
+                "@vtotal, " \
+                "@vbend, " \
+                "@vbstart);"
 
 #define SQL_INSERT_CONTROL \
         "INSERT INTO control VALUES (" \
@@ -352,9 +374,13 @@ static struct
         const gchar *flipx;
         const gchar *game;
         const gchar *graphic;
+        const gchar *hbend;
+        const gchar *hbstart;
         const gchar *height;
+        const gchar *htotal;
         const gchar *index_;
         const gchar *input;
+        const gchar *isbios;
         const gchar *keydelta;
         const gchar *mame;
         const gchar *manufacturer;
@@ -366,6 +392,7 @@ static struct
         const gchar *offset;
         const gchar *orientation;
         const gchar *palettesize;
+        const gchar *pixclock;
         const gchar *players;
         const gchar *protection;
         const gchar *refresh;
@@ -388,6 +415,9 @@ static struct
         const gchar *status;
         const gchar *tilt;
         const gchar *type;
+        const gchar *vbend;
+        const gchar *vbstart;
+        const gchar *vtotal;
         const gchar *width;
         const gchar *year;
 
@@ -643,6 +673,20 @@ db_parser_start_element_display (ParserData *data,
                         param = "@height";
                 else if (attribute_name[ii] == intern.refresh)
                         param = "@refresh";
+                else if (attribute_name[ii] == intern.pixclock)
+                        param = "@pixclock";
+                else if (attribute_name[ii] == intern.htotal)
+                        param = "@htotal";
+                else if (attribute_name[ii] == intern.hbend)
+                        param = "@hbend";
+                else if (attribute_name[ii] == intern.hbstart)
+                        param = "@hbstart";
+                else if (attribute_name[ii] == intern.vtotal)
+                        param = "@vtotal";
+                else if (attribute_name[ii] == intern.vbend)
+                        param = "@vbend";
+                else if (attribute_name[ii] == intern.vbstart)
+                        param = "@vbstart";
                 else
                         continue;
 
@@ -695,9 +739,13 @@ db_parser_start_element_game (ParserData *data,
                               GError **error)
 {
         sqlite3_stmt *stmt = data->insert_game_stmt;
+        gchar *category;
+        gint error_code;
         gint ii;
+        GError *local_error = NULL;
 
         /* Bind default values. */
+        db_parser_bind_text (stmt, "@isbios", "no");
         db_parser_bind_text (stmt, "@runnable", "yes");
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
@@ -711,6 +759,8 @@ db_parser_start_element_game (ParserData *data,
                 }
                 else if (attribute_name[ii] == intern.sourcefile)
                         param = "@sourcefile";
+                else if (attribute_name[ii] == intern.isbios)
+                        param = "@isbios";
                 else if (attribute_name[ii] == intern.runnable)
                         param = "@runnable";
                 else if (attribute_name[ii] == intern.cloneof)
@@ -724,6 +774,22 @@ db_parser_start_element_game (ParserData *data,
 
                 db_parser_bind_text (stmt, param, attribute_value[ii]);
         }
+
+#ifdef CATEGORY_FILE
+        /* Lookup category from the catver.ini file. */
+        g_return_if_fail (data->game != NULL);
+        category = gva_categories_lookup (data->game, &local_error);
+
+        /* Slience "key not found" errors. */
+        error_code = G_KEY_FILE_ERROR_KEY_NOT_FOUND;
+        if (g_error_matches (local_error, G_KEY_FILE_ERROR, error_code))
+                g_clear_error (&local_error);
+
+        if (category != NULL)
+                db_parser_bind_text (stmt, "@category", category);
+        else if (local_error != NULL)
+                g_propagate_error (error, local_error);
+#endif
 }
 
 static void
@@ -904,7 +970,15 @@ db_parser_start_element (GMarkupParseContext *context,
 
         /* XXX Copied from below... */
 
-        if (element_name == intern.driver)
+        if (element_name == intern.chip)
+                db_parser_start_element_chip (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.display)
+                db_parser_start_element_display (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.driver)
                 db_parser_start_element_driver (
                         data, attribute_name, attribute_value, error);
 
@@ -1015,7 +1089,13 @@ db_parser_end_element (GMarkupParseContext *context,
 
         /* XXX Copied from below... */
 
-        if (element_name == intern.game)
+        if (element_name == intern.chip)
+                db_parser_exec_stmt (data->insert_chip_stmt, error);
+
+        else if (element_name == intern.display)
+                db_parser_exec_stmt (data->insert_display_stmt, error);
+
+        else if (element_name == intern.game)
                 db_parser_end_element_game (data, error);
 
         /* Skip unused elements to speed up parsing. */
@@ -1217,40 +1297,6 @@ db_create_tables (GError **error)
                 && gva_db_execute (SQL_CREATE_VIEW_AVAILABLE, error);
 }
 
-static void
-db_function_getcategory (sqlite3_context *context,
-                         gint n_values,
-                         sqlite3_value **values)
-{
-#ifdef CATEGORY_FILE
-        const gchar *name;
-        gchar *category;
-        gint error_code;
-        GError *error = NULL;
-
-        g_assert (n_values == 1);
-
-        name = (const gchar *) sqlite3_value_text (values[0]);
-        category = gva_categories_lookup (name, &error);
-
-        /* Silence "key not found" errors. */
-        error_code = G_KEY_FILE_ERROR_KEY_NOT_FOUND;
-        if (g_error_matches (error, G_KEY_FILE_ERROR, error_code))
-                g_clear_error (&error);
-
-        gva_error_handle (&error);
-
-        if (category != NULL)
-                sqlite3_result_text (context, category, -1, SQLITE_TRANSIENT);
-        else
-                sqlite3_result_null (context);
-
-        g_free (category);
-#else
-        sqlite3_result_null (context);
-#endif
-}
-
 static void
 db_function_isfavorite (sqlite3_context *context,
                         gint n_values,
@@ -1267,6 +1313,39 @@ db_function_isfavorite (sqlite3_context *context,
                 sqlite3_result_text (context, "no", -1, SQLITE_STATIC);
 }
 
+static void
+db_function_match (sqlite3_context *context,
+                   gint n_values,
+                   sqlite3_value **values)
+{
+        const gchar *text1;
+        const gchar *text2;
+        gboolean match = FALSE;
+
+        g_assert (n_values == 2);
+
+        /* XXX Note the operands are not in the order one might think.
+         *     Still trying to find some documentation to confirm this. */
+
+        text1 = (const gchar *) sqlite3_value_text (values[1]);
+        text2 = (const gchar *) sqlite3_value_text (values[0]);
+
+        if (text1 != NULL && text2 != NULL)
+        {
+                gchar *s1, *s2;
+
+                s1 = gva_search_collate_key (text1);
+                s2 = gva_search_collate_key (text2);
+
+                match = (strstr (s1, s2) != NULL);
+
+                g_free (s1);
+                g_free (s2);
+        }
+
+        sqlite3_result_int (context, match);
+}
+
 static void
 db_trace_cb (gpointer unused, const gchar *message)
 {
@@ -1307,8 +1386,8 @@ gva_db_init (GError **error)
                 goto fail;
 
         errcode = sqlite3_create_function (
-                db, "getcategory", 1, SQLITE_ANY, NULL,
-                db_function_getcategory, NULL, NULL);
+                db, "match", 2, SQLITE_ANY, NULL,
+                db_function_match, NULL, NULL);
         if (errcode != SQLITE_OK)
                 goto fail;
 
@@ -1369,9 +1448,13 @@ gva_db_build (GError **error)
         intern.flipx        = g_intern_static_string ("flipx");
         intern.game         = g_intern_static_string ("game");
         intern.graphic      = g_intern_static_string ("graphic");
+        intern.hbend        = g_intern_static_string ("hbend");
+        intern.hbstart      = g_intern_static_string ("hbstart");
         intern.height       = g_intern_static_string ("height");
+        intern.htotal       = g_intern_static_string ("htotal");
         intern.index_       = g_intern_static_string ("index_");
         intern.input        = g_intern_static_string ("input");
+        intern.isbios       = g_intern_static_string ("isbios");
         intern.keydelta     = g_intern_static_string ("keydelta");
         intern.mame         = g_intern_static_string ("mame");
         intern.manufacturer = g_intern_static_string ("manufacturer");
@@ -1383,6 +1466,7 @@ gva_db_build (GError **error)
         intern.offset       = g_intern_static_string ("offset");
         intern.orientation  = g_intern_static_string ("orientation");
         intern.palettesize  = g_intern_static_string ("palettesize");
+        intern.pixclock     = g_intern_static_string ("pixclock");
         intern.players      = g_intern_static_string ("players");
         intern.protection   = g_intern_static_string ("protection");
         intern.refresh      = g_intern_static_string ("refresh");
@@ -1405,6 +1489,9 @@ gva_db_build (GError **error)
         intern.status       = g_intern_static_string ("status");
         intern.tilt         = g_intern_static_string ("tilt");
         intern.type         = g_intern_static_string ("type");
+        intern.vbend        = g_intern_static_string ("vbend");
+        intern.vbstart      = g_intern_static_string ("vbstart");
+        intern.vtotal       = g_intern_static_string ("vtotal");
         intern.width        = g_intern_static_string ("width");
         intern.year         = g_intern_static_string ("year");
 
