@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -21,6 +21,8 @@
 #include <string.h>
 #include <time.h>
 
+#include "gva-columns.h"
+#include "gva-db.h"
 #include "gva-error.h"
 #include "gva-favorites.h"
 #include "gva-time.h"
@@ -120,8 +122,11 @@ game_store_constructor (GType type,
         gint column = 0;
 
         types[column++] = G_TYPE_STRING;     /* COLUMN_NAME */
+        types[column++] = G_TYPE_STRING;     /* COLUMN_BIOS */
+        types[column++] = G_TYPE_STRING;     /* COLUMN_CATEGORY */
         types[column++] = G_TYPE_BOOLEAN;    /* COLUMN_FAVORITE */
         types[column++] = G_TYPE_STRING;     /* COLUMN_SOURCEFILE */
+        types[column++] = G_TYPE_BOOLEAN;    /* COLUMN_ISBIOS */
         types[column++] = G_TYPE_BOOLEAN;    /* COLUMN_RUNNABLE */
         types[column++] = G_TYPE_STRING;     /* COLUMN_CLONEOF */
         types[column++] = G_TYPE_STRING;     /* COLUMN_ROMOF */
@@ -146,6 +151,8 @@ game_store_constructor (GType type,
         types[column++] = G_TYPE_STRING;     /* COLUMN_DRIVER_PROTECTION */
         types[column++] = G_TYPE_STRING;     /* COLUMN_DRIVER_SAVESTATE */
         types[column++] = G_TYPE_INT;        /* COLUMN_DRIVER_PALETTESIZE */
+        types[column++] = G_TYPE_STRING;     /* COLUMN_COMMENT */
+        types[column++] = G_TYPE_INT64;      /* COLUMN_INODE */
         types[column++] = G_TYPE_STRING;     /* COLUMN_INPFILE */
         types[column++] = GVA_TYPE_TIME;     /* COLUMN_TIME */
 
@@ -155,8 +162,8 @@ game_store_constructor (GType type,
         object = G_OBJECT_CLASS (parent_class)->constructor (
                 type, n_construct_properties, construct_properties);
 
-        gtk_list_store_set_column_types (
-                GTK_LIST_STORE (object), G_N_ELEMENTS (types), types);
+        gtk_tree_store_set_column_types (
+                GTK_TREE_STORE (object), G_N_ELEMENTS (types), types);
 
         gtk_tree_sortable_set_default_sort_func (
                 GTK_TREE_SORTABLE (object),
@@ -220,18 +227,41 @@ gva_game_store_get_type (void)
                 };
 
                 type = g_type_register_static (
-                        GTK_TYPE_LIST_STORE, "GvaGameStore", &type_info, 0);
+                        GTK_TYPE_TREE_STORE, "GvaGameStore", &type_info, 0);
         }
 
         return type;
 }
 
+/**
+ * gva_game_store_new:
+ *
+ * Creates a new #GvaGameStore with pre-defined columns and settings.
+ *
+ * Returns: a new #GvaGameStore
+ **/
 GtkTreeModel *
 gva_game_store_new (void)
 {
         return g_object_new (GVA_TYPE_GAME_STORE, NULL);
 }
 
+/**
+ * gva_game_store_new_from_query:
+ * @sql: an SQL query
+ * @error: return locations for a #GError, or %NULL
+ *
+ * This may be the most powerful function in
+ * <emphasis>GNOME Video Arcade</emphasis>.
+ *
+ * Creates a new #GvaGameStore by executing the given SQL query on the games
+ * database and converting the results to tree model rows.  The resulting
+ * #GtkTreeModel can then be plugged into a #GtkTreeView.
+ *
+ * XXX Say more here.
+ *
+ * Returns: a new #GvaGameStore
+ **/
 GtkTreeModel *
 gva_game_store_new_from_query (const gchar *sql,
                                GError **error)
@@ -297,8 +327,8 @@ gva_game_store_new_from_query (const gchar *sql,
                 GtkTreeIter iter;
                 GValue *value;
 
-                /* Append a new row to the list store. */
-                gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+                /* Append a new row to the tree store. */
+                gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
 
                 /* Populate the row with available values. */
                 for (ii = 0; ii < n_columns; ii++)
@@ -313,7 +343,8 @@ gva_game_store_new_from_query (const gchar *sql,
                                 const gchar *text;
                                 gboolean v_boolean;
 
-                                text = sqlite3_column_text (stmt, ii);
+                                text = (const gchar *)
+                                        sqlite3_column_text (stmt, ii);
                                 v_boolean = (text != NULL) &&
                                         (strcmp (text, "yes") == 0);
                                 g_value_set_boolean (value, v_boolean);
@@ -329,7 +360,8 @@ gva_game_store_new_from_query (const gchar *sql,
                         {
                                 const gchar *v_string;
 
-                                v_string = sqlite3_column_text (stmt, ii);
+                                v_string = (const gchar *)
+                                        sqlite3_column_text (stmt, ii);
                                 if (v_string == NULL)
                                         v_string = "";
                                 g_value_set_string (value, v_string);
@@ -339,17 +371,17 @@ gva_game_store_new_from_query (const gchar *sql,
                                 g_assert_not_reached ();
                         }
 
-                        gtk_list_store_set_value (
-                                GTK_LIST_STORE (model), &iter,
+                        gtk_tree_store_set_value (
+                                GTK_TREE_STORE (model), &iter,
                                 column_ids[ii], value);
                 }
 
-                name = sqlite3_column_text (stmt, name_column);
+                name = (const gchar *) sqlite3_column_text (stmt, name_column);
 
                 value = &column_values[n_columns];
                 g_value_set_boolean (value, gva_favorites_contains (name));
-                gtk_list_store_set_value (
-                        GTK_LIST_STORE (model), &iter,
+                gtk_tree_store_set_value (
+                        GTK_TREE_STORE (model), &iter,
                         GVA_GAME_STORE_COLUMN_FAVORITE, value);
 
                 /* Add an entry for this row to the index. */
@@ -357,7 +389,8 @@ gva_game_store_new_from_query (const gchar *sql,
                         GVA_GAME_STORE (model), g_strdup (name), &iter);
 
                 /* Keep the UI responsive. */
-                g_main_context_iteration (NULL, FALSE);
+                if (gtk_main_iteration_do (FALSE))
+                        goto fail;
         }
 
         /* Query complete. */
@@ -381,15 +414,31 @@ exit:
         return model;
 }
 
+/**
+ * gva_game_store_clear:
+ * @game_store: a #GvaGameStore
+ *
+ * Removes all rows from @game_store and clears the internal index.
+ **/
 void
 gva_game_store_clear (GvaGameStore *game_store)
 {
         g_return_if_fail (GVA_IS_GAME_STORE (game_store));
 
         g_hash_table_remove_all (game_store_get_index (game_store));
-        gtk_list_store_clear (GTK_LIST_STORE (game_store));
+        gtk_tree_store_clear (GTK_TREE_STORE (game_store));
 }
 
+/**
+ * gva_game_store_index_insert:
+ * @game_store: a #GvaGameStore
+ * @key: an index key
+ * @iter: a #GtkTreeIter pointing to a row in @game_store
+ *
+ * Adds an entry to @game_store's internal index.  You will want to call
+ * this immediately after adding a new row to @game_store, such as with
+ * gtk_tree_store_append().
+ **/
 void
 gva_game_store_index_insert (GvaGameStore *game_store,
                              const gchar *key,
@@ -412,6 +461,16 @@ gva_game_store_index_insert (GvaGameStore *game_store,
         gtk_tree_path_free (path);
 }
 
+/**
+ * gva_game_store_index_lookup:
+ * @game_store: a #GvaGameStore
+ * @key: an index key
+ *
+ * Looks up the row corresponding to @key in @game_store and returns a
+ * #GtkTreePath to it, or %NULL if the row was not found.
+ *
+ * Returns: a #GtkTreePath to the row corresponding to @key, or %NULL
+ **/
 GtkTreePath *
 gva_game_store_index_lookup (GvaGameStore *game_store,
                              const gchar *key)
