@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -16,7 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gva-common.h"
+#include "gva-history.h"
 
 #include "gva-error.h"
 
@@ -39,10 +39,9 @@ history_file_open (GError **error)
                 g_assert (status == G_IO_STATUS_NORMAL);
         }
 #else
-        g_set_error (
-                error, GVA_ERROR, GVA_ERROR_CONFIG,
-                _("This program is not configured to show "
-                  "arcade history information."));
+        g_message (
+                _("This program is not configured "
+                  "to show history information."));
 #endif
 
         return channel;
@@ -137,9 +136,9 @@ gva_history_init (GError **error)
  * @error: return location for a #GError, or %NULL
  *
  * Returns arcade history information for @game.  If an error occurs,
- * it returns %FALSE and sets @error.
+ * it returns %NULL and sets @error.
  *
- * Returns: %TRUE on success, %FALSE if an error occurred
+ * Returns: history for @game, or %NULL if an error occurred
  **/
 gchar *
 gva_history_lookup (const gchar *game,
